@@ -464,4 +464,15 @@ describe('WorkerHandler', function () {
       });
     });
   });
+
+  describe('workerAlreadyTerminated', function () {
+    it('worker handler checks if terminated before handling message', function (done) {
+      var handler = new WorkerHandler();
+      const worker = handler.worker;
+      handler.terminate(true, () => {
+        worker.emit('message', 'ready');
+        done();
+      });
+    });
+  });
 });
